const { readFileSync, writeFileSync, readdirSync } = require('fs');
const { join } = require('path');

const entryRegex = /exports\.transform\s*=\s*function\s*\(model\)\s*\{/;
const testDataRegex = /(?:\/\/)*\s*model.testData\s*=[^\r\n]*\r?\n/g;
const clientScriptRegex = /(?:\r?\n)*<!-- debug -->(.|\r|\n)*<!-- \/debug -->/gm;
const contentWrapperRegex = /<!-- content -->\r?\n|\r?\n<!-- \/content -->/g;

const clientScript = `
<!-- debug -->
<script>window.__contentTemplateInfo = {{{testData}}};</script>
<script src="http://localhost:5555/socket.io/socket.io.js"></script>
<script src="http://localhost:5555/client.js"></script>
<!-- /debug -->`;

const contentTemplates = join(process.cwd(), 'src', 'ContentTemplate');

module.exports = function (enable) {
    readdirSync(contentTemplates).filter(f => /\.html\.primary\.js$/.test(f)).forEach(name => {
        const templateName = name.substr(0, name.length - '.html.primary.js'.length);
        const filename = join(contentTemplates, name);
        let content = readFileSync(filename).toString()
            .replace(testDataRegex, '');
        if (enable) {
            content = content.replace(entryRegex, `$&\n\tmodel.testData = JSON.stringify({ model: model, name: '${templateName}' });\r\n`);
        }
        writeFileSync(filename, content, { flag: 'w' });

        const tmplFilename = join(contentTemplates, templateName + '.html.primary.tmpl');
        let tmplContent = readFileSync(tmplFilename).toString()
            .replace(clientScriptRegex, '')
            .replace(contentWrapperRegex, '');
        if (enable) {
            tmplContent = '<!-- content -->\r\n' + tmplContent + clientScript + '\r\n<!-- /content -->';
        }
        writeFileSync(tmplFilename, tmplContent, { flag: 'w' });
    });
};