const edge = require('edge');
const { readFileSync, writeFileSync, readdirSync, mkdirSync } = require('fs');
const { join } = require('path');

const docfxVersion = '2.23.1';

const references = [
    join(__dirname, `../nuget_packages/Microsoft.DocAsCode.Common.${docfxVersion}/lib/net46/Microsoft.DocAsCode.Common.dll`),
    join(__dirname, `../nuget_packages/Microsoft.DocAsCode.Plugins.${docfxVersion}/lib/net46/Microsoft.DocAsCode.Plugins.dll`),
    join(__dirname, `../nuget_packages/Microsoft.DocAsCode.Build.Engine.${docfxVersion}/lib/net46/Microsoft.DocAsCode.Build.Engine.dll`),
    join(__dirname, '../nuget_packages/System.Collections.Immutable.1.3.1/lib/netstandard1.0/System.Collections.Immutable.dll'),
    join(__dirname, '../nuget_packages/Jint.2.5.0/lib/portable-net40+sl50+win+wpa81+wp80/Jint.dll'),
    join(__dirname, '../nuget_packages/Nustache.1.15.3.5/lib/net20/Nustache.Core.dll'),
    join(__dirname, '../nuget_packages/Newtonsoft.Json.9.0.1/lib/net45/Newtonsoft.Json.dll'),
];

const source = `
    using System;
    using System.Threading.Tasks;
    using Microsoft.DocAsCode.Build.Engine;

    public class Startup
    {
        public async Task<object> Invoke(object input)
        {
            var x = (dynamic)input;

            var docfxCommon = new FileResourceCollection(x.docfxCommon);
            var contentTemplates = new FileResourceCollection(x.contentTemplates);
            var resourceCollection = new CompositeResourceCollectionWithOverridden(new ResourceCollection[] { contentTemplates, docfxCommon });
            var context = new DocumentBuildContext(x.emptyFolder);
            var scriptResource = new TemplatePreprocessorResource(x.transform.resourceName, x.transform.content);
            var preprocessor = new TemplateJintPreprocessor(resourceCollection, scriptResource, context);
            var transformedModel = preprocessor.TransformModelFunc(x.model);
            var templateRendererResource = new TemplateRendererResource(x.template.resourceName, x.template.content, x.template.name);
            var rendererType = typeof(FileResourceCollection).Assembly.GetType("Microsoft.DocAsCode.Build.Engine.MustacheTemplateRenderer");
            var renderer = Activator.CreateInstance(rendererType, resourceCollection, templateRendererResource);
            return rendererType.GetMethod("Render").Invoke(renderer, new object[] { transformedModel });
        }
    }`;

const render = edge.func({ source, references });

try { mkdirSync(join(__dirname, 'empty-folder')); } catch(err) {}
const emptyFolder = join(__dirname, 'empty-folder');

const docfxCommon = join(__dirname, `..\\docfx\\src\\docfx.website.themes\\common`);
const contentTemplates = join(process.cwd(), 'src', 'ContentTemplate');

module.exports = function(model, name) {
    const transform = {
        resourceName: name + '.html.primary.js',
        content: readFileSync(join(contentTemplates, name + '.html.primary.js')).toString()
    };

    const template = {
        name,
        resourceName: name + '.html.primary.tmpl',
        content: readFileSync(join(contentTemplates, name + '.html.primary.tmpl' )).toString()
    };

    const html = render({ emptyFolder, model, contentTemplates, docfxCommon, transform, template }, true);

    return html;
};
