const { readFileSync, writeFileSync } = require('fs');
const { createServer } = require('http');
const { join } = require('path');
const socketio = require('socket.io');
const chokidar = require('chokidar');
const render = require('./render');

module.exports = function () {
    const clientjs = readFileSync(join(__dirname, 'client.js')).toString();
    const server = createServer((req, res) => {
        if (req.method === 'POST' && req.url === '/') {
            const body = [];
            req.on('data', chunk => body.push(chunk))
                .on('error', err => console.log(err))
                .on('end', () => {
                    res.on('error', err => console.log(err));
                    res.setHeader('Access-Control-Allow-Origin', req.headers.origin);
                    let result = 'success';
                    try {
                        renderTemplate(JSON.parse(Buffer.concat(body).toString()));
                        res.writeHead(200);
                    } catch (err) {
                        console.log(err);
                        result = err.toString();
                        res.writeHead(500);
                    }
                    res.end(result);
                });
            return;
        }

        if (req.method === 'GET' && req.url === '/client.js') {
            res.writeHead(200, { 'Content-Type': 'application/javascript' });
            res.end(clientjs);
            return;
        }

        res.writeHead(404);
    });

    const io = socketio(server);
    startWatch(() => io.emit('content-templates-changed'));

    server.listen(5555);
};

const contentRegex = /<!-- content -->(?:.|\r|\n)*<!-- \/content -->/g;

function startWatch(callback) {
    // const project = require(join(process.cwd(), 'project.json'));
    // const opConfig = require(join(process.cwd(), project.testdata, '.openpublishing.publish.config.json'));
    // const foldersToWatch = opConfig.docsets_to_publish
    //     .map(d => join(process.cwd(), project.testdata, d.build_source_folder, '**/*.{md,yml}'));
    // foldersToWatch.push(join(process.cwd(), 'src/ContentTemplate/**/*'));
    // for (const f of foldersToWatch) {
    //     console.log(`docfx-dev watching: ${f}`);
    // }
    chokidar.watch(join(process.cwd(), 'src/ContentTemplate/**/*')).on('all', callback);
}

function renderTemplate(info) {
    const html = render(info.model, info.templateName);
    const path = info.pathname.substr('/en-us/'.length);
    const filename = join(process.cwd(), 'drop', 'site', path);
    const content = readFileSync(filename).toString()
        .replace(contentRegex, html);
    writeFileSync(filename, content, { flag: 'w' });
    console.log(`Rendered content for "${path}" using "${info.templateName}" template.`);
}

    // if (project.testDataFormat !== 'op') {
    //   callback('cannot run opbuild when testDataFormat is not op.');
    //   return;
    // }
    // var absoluteContentRoot = project.config.testdata;
    // console.log('starting opbuild on:' + absoluteContentRoot);
    // if (!_path2.default.isAbsolute(project.config.testdata)) {
    //   // check if it is a valid content git repo
    //   absoluteContentRoot = _path2.default.resolve(project.directory, project.config.testdata);
    // }
    // var outputPath = _path2.default.join(project.directory, project.drop + '/site');
    // var opScript = absoluteContentRoot + '\\.openpublishing.build.ps1 -parameters:\'OutputFolder=' + outputPath + ';Targets=LocalBuild\'';
    // var execProcess = (0, _child_process.exec)('powershell -ExecutionPolicy Bypass ' + opScript, { cwd: absoluteContentRoot });

    // execProcess.stdout.on('data', function (data) {
    //   console.log(data);
    // });
    // execProcess.on('close', function (code) {
    //   callback(!!code);
    // });
